// A ColorCreator is used to create any color


import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.*;


public class ColorCreator extends JPanel implements ChangeListener
{
// Static variables

  public static final int HORIZONTAL = 1,
                          VERTICAL = 2;
                          
  private static Color defaultColor = Color.green;
  private static int defaultAlignment = ColorCreator.HORIZONTAL;


// Protected members which contain key information

  protected Color currentColor;
  protected int alignment;

  
// Other member variables

  private ColorPanel colorPanel;

  private JButton buttonArray[];

  private Color purpleColor = new Color(255, 0, 255),
                buttonColors[] = {Color.red,   Color.yellow, Color.blue,
                                  Color.black, Color.gray,   Color.white,
                                  purpleColor, Color.orange, Color.green};
  private String buttonNames[] = {"Red",       "Yellow",     "Blue",
                                  "Black",     "Random",     "White",
                                  "Purple",    "Orange",     "Green"};
  private final int numberOfButtons = buttonColors.length;

  private JSlider sliderArray[];

  private JTextField textFieldArray[];


// Constructors

  public ColorCreator()
  {
    this(ColorCreator.defaultAlignment, ColorCreator.defaultColor);
  }

  public ColorCreator(int alignment)
  {
    this(alignment, ColorCreator.defaultColor);
  }

  public ColorCreator(Color currentColor)
  {
    this(ColorCreator.defaultAlignment, currentColor);
  }

  public ColorCreator(int alignment, Color currentColor)
  {
    createComponents();
    addComponents();
    setAlignment(alignment);
    update(currentColor);
  }


// public methods

  public Color getCurrentColor()
  {
    return currentColor;
  }

  public void setCurrentColor(Color currentColor)
  {
    this.currentColor = currentColor;
  }

  public int getAlignment()
  {
    return alignment;
  }

  public void setAlignment(int alignment)
  {
    this.alignment = alignment;
    if (alignment == ColorCreator.HORIZONTAL) {
      setLayout(new GridLayout(1,2));
    }
    else {
      setLayout(new GridLayout(2,1));
    }
    validate();
  }


// Methods for creating the ColorCreator.

  private void createComponents()
  {
	colorPanel = new ColorPanel();
    buttonArray = new JButton[numberOfButtons];
	ButtonHandler buttonHandler = new ButtonHandler();
    for (int i = 0; i < numberOfButtons; i++) {
      buttonArray[i] = new JButton();
      buttonArray[i].setBackground(buttonColors[i]);
      buttonArray[i].setActionCommand(buttonNames[i]);
      buttonArray[i].addActionListener(buttonHandler);
      if ( buttonArray[i].getActionCommand() == "Random" )
        buttonArray[i].setText("?");
    }
  
    textFieldArray = new JTextField[3];
    TextFieldHandler fieldListener = new TextFieldHandler();
    sliderArray = new JSlider[3];
    
    for (int i = 0; i < 3; i++) {
      textFieldArray[i] = new JTextField(3);
      textFieldArray[i].setBackground(Color.white);
      textFieldArray[i].addActionListener(fieldListener);

      sliderArray[i] = new JSlider(JSlider.HORIZONTAL, 0, 255, 100);
      sliderArray[i].addChangeListener(this);
    }
  }

  private void addComponents()
  {
 
	JPanel sliderTextFieldPanel = new JPanel(new GridLayout(3,1)),
           sliderTextFieldPanels[] = new JPanel[3];
    for (int i = 0; i < 3; i++)
      sliderTextFieldPanels[i] = new JPanel(new BorderLayout());
    for (int i = 0; i < 3; i++) 
	{
      sliderTextFieldPanel.add(sliderTextFieldPanels[i]);
      sliderTextFieldPanels[i].add(sliderArray[i], BorderLayout.CENTER);
      sliderTextFieldPanels[i].add(textFieldArray[i], BorderLayout.EAST);
    }

    JPanel colorButtonPanel = new JPanel( new GridLayout(1,2) ),  
           buttonPanel = new JPanel(new GridLayout(3,3));
    colorButtonPanel.add(colorPanel);
    colorButtonPanel.add(buttonPanel);
    for (int i = 0; i < numberOfButtons; i++) 
      buttonPanel.add(buttonArray[i]);

	add(sliderTextFieldPanel);  // The alignment is set
    add(colorButtonPanel);      // elsewhere (it's a GridLayout).
  }


 // Event handling methods.

  private class TextFieldHandler implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
	{
      update(getTextFieldValuesColor());
	}
  } 

  private class ButtonHandler implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
	{
      String event = e.getActionCommand();
      if (event == "Random")
        update( getRandomColor() );
	  else
        for (int i = 0; i < numberOfButtons; i++)
          if (event == buttonNames[i])
            update( buttonColors[i] );
	}
  }


  public void stateChanged(ChangeEvent e)
  {
    int colors[] = new int[3];
    for (int i = 0; i < 3; i++) 
      colors[i] = sliderArray[i].getValue();
    update(new Color(colors[0], colors[1], colors[2]));
  }


 // Other methods

  private void update(Color color)
  {
    currentColor = color;
    int colors[] = new int[3];
    colors[0] = color.getRed();
    colors[1] = color.getGreen();
    colors[2] = color.getBlue();

    for (int i = 0; i < 3; i++) {
      sliderArray[i].setValue(colors[i]);
      textFieldArray[i].setText(String.valueOf(colors[i]));
    }

    colorPanel.setBackground(currentColor);
    colorPanel.repaint();

  } 

  private Color getTextFieldValuesColor()
  {
    int colors[] = new int[3];
    for (int i = 0; i < 3; i++) {
      String string = textFieldArray[i].getText();
      try {
        colors[i] = Integer.parseInt(string);
      }
      catch ( Exception e)
      {
        colors[i] = 0;
      }
      if (colors[i] < 0)
        colors[i] = 0;
      else if (colors[i] > 255)
        colors[i] = 255;
    }
    return ( new Color( colors[0], colors[1], colors[2] ) );
  }

  private int getRandomInt()
  {
    return ( (int) (256*Math.random()) );
  }

  private Color getRandomColor()
  {
    return ( new Color( getRandomInt(), getRandomInt(), getRandomInt() ) );
  }

  private class ColorPanel extends JPanel  // Used to display current color
  {
    public void paintComponent(Graphics g)
	{
	  super.paintComponent(g);

	  int width = getSize().width,          // Simply
          height = getSize().height;        // paints  
      g.setColor(Color.black);              // a frame
      g.drawRect(0, 0, width-1, height-1);  // around
      g.drawRect(1, 1, width-3, height-3);  // Panel.
	}
  }

} // end class ColorCreator


